@@ -272,12 +272,15 @@
     !! This routine solves the isotropic Eliashberg equations on the imaginary-axis
     !!
     !! SH: Modified to allow for fbw calculations (Nov 2021).
+    !! SM: Added paralleilization of freq. among images (April 2025).
     !
     USE kinds,         ONLY : DP
     USE io_global,     ONLY : stdout
     USE input,         ONLY : nsiter, muc, conv_thr_iaxis, &
                               fbw, dos_del, muchem
     USE global_var,    ONLY : gtemp
+    USE mp_global,     ONLY : inter_image_comm, my_image_id
+    USE mp,            ONLY : mp_sum
     USE supercond_common,     ONLY : nsiw, gap0, wsi, wsn, keri, muintr, &
                               deltai, deltaip, znormi, nznormi, &
                               znormip, shifti, shiftip, ef0, &
@@ -302,6 +305,8 @@
     !! Error status
     INTEGER :: ie
     !! Counter on energy grid
+    INTEGER :: startiw, lastiw
+    !! Lower/upper bound index for iw-points
     !
     REAL(KIND = DP) :: lambdam
     !! K_{-}(n,n',T))
@@ -328,7 +333,14 @@
     REAL(KIND = DP), ALLOCATABLE, SAVE :: deltaold(:)
     !! supercond. gap from previous iteration
     !
+    ! SM: added image parallelization for distributing freq. points into images
+    CALL divide(inter_image_comm, nsiw(itemp), startiw, lastiw) 
+    !
     IF (iter == 1) THEN
+      ! Print for user information
+      WRITE(stdout, '(5x, "   startiw = ", i0, ", lastiw = ", i0, ", nsiw(itemp) = ", i0)') &
+      startiw, lastiw, nsiw(itemp)
+      !
       ALLOCATE(inv_wsi(nsiw(itemp)), STAT = ierr)
       IF (ierr /= 0) CALL errore('sum_eliashberg_iso_iaxis', 'Error allocating inv_wsi', 1)
       !
@@ -401,7 +413,9 @@
         desqrt = desqrt * inv_dos
         sesqrt = sesqrt * inv_dos
         !
-        DO iw = 1, nsiw(itemp) ! loop over omega
+        !DO iw = 1, nsiw(itemp) ! loop over omega
+        ! SM: distribute into images
+        DO iw = startiw, lastiw
           ! SH: For general case (including sparse sampling)
           !       "actual" matsubara indices n1/n2 are needed instead of iw/iwp
           lambdam = keri(ABS(wsn(iw) - wsn(iwp)) + 1)
@@ -419,8 +433,8 @@
         ENDDO ! iw
       ENDDO ! iwp
       !
-      absdelta   = zero
-      reldelta   = zero
+      absdelta = zero
+      reldelta = zero
       DO iw = 1, nsiw(itemp) ! loop over omega
         nznormi(iw) = 1.d0 + gtemp(itemp) * nznormi(iw) * inv_wsi(iw)
         ! Eqs.(34)-(35) in Margine and Giustino, PRB 87, 024505 (2013)
@@ -430,7 +444,14 @@
         reldelta   = reldelta + ABS(deltai(iw) - deltaold(iw))
         absdelta   = absdelta + ABS(deltai(iw))
       ENDDO ! iw
-    ELSE
+      ! Collect from images
+      CALL mp_sum(nznormi, inter_image_comm)
+      CALL mp_sum(znormi, inter_image_comm)
+      CALL mp_sum(deltai, inter_image_comm)
+      CALL mp_sum(shifti, inter_image_comm)
+      CALL mp_sum(reldelta, inter_image_comm)
+      CALL mp_sum(absdelta, inter_image_comm)
+    ELSE !FSR
       deltai(:)  = zero
       znormi(:)  = zero
       nznormi(:) = zero
@@ -458,7 +479,9 @@
       !
       absdelta = zero
       reldelta = zero
-      DO iw = 1, nsiw(itemp) ! loop over omega
+      !DO iw = 1, nsiw(itemp) ! loop over omega
+      !SM: distribute into images
+      DO iw = startiw, lastiw
         znormi(iw)  = 1.d0 + pi * gtemp(itemp) * znormi(iw) * inv_wsi(iw)
         ! Eqs.(34)-(35) in Margine and Giustino, PRB 87, 024505 (2013)
         nznormi(iw) = 1.d0 + pi * gtemp(itemp) * nznormi(iw) * inv_wsi(iw)
@@ -466,6 +489,12 @@
         reldelta = reldelta + ABS(deltai(iw) - deltaold(iw))
         absdelta = absdelta + ABS(deltai(iw))
       ENDDO ! iw
+      ! Collect from images
+      CALL mp_sum(nznormi, inter_image_comm)
+      CALL mp_sum(znormi, inter_image_comm)
+      CALL mp_sum(deltai, inter_image_comm)
+      CALL mp_sum(reldelta, inter_image_comm)
+      CALL mp_sum(absdelta, inter_image_comm)
     ENDIF ! fbw
     errdelta = reldelta / absdelta
     deltaold(:) = deltai(:)
@@ -479,9 +508,9 @@
         iter, errdelta, znormi(1), deltai(1) * 1000.d0, shifti(1) * 1000.d0, muintr
     ELSE
       WRITE(stdout, '(5x, i6, 3ES15.6)') iter, errdelta, znormi(1), deltai(1) * 1000.d0
-!     WRITE(stdout, '(5x, a, i6, a, ES15.6, a, ES15.6, a, ES15.6)') 'iter = ', iter, &
-!                  '   ethr = ', errdelta, '   znormi(1) = ', znormi(1), &
-!                  '   deltai(1) = ', deltai(1)
+      !WRITE(stdout, '(5x, a, i6, a, ES15.6, a, ES15.6, a, ES15.6)') 'iter = ', iter, &
+      !            '   ethr = ', errdelta, '   znormi(1) = ', znormi(1), &
+      !            '   deltai(1) = ', deltai(1)
     ENDIF
     !
     IF (errdelta < conv_thr_iaxis) conv = .TRUE.
@@ -550,7 +579,7 @@
     USE io_global,     ONLY : stdout
     USE input,         ONLY : nqstep, nsiter, conv_thr_racon, lpade
     USE global_var,    ONLY : gtemp
-    USE supercond_common,     ONLY : nsw, dwsph, ws, gap0, a2f_iso, dsumi, zsumi, &
+    USE supercond_common,     ONLY : nsw, dwsph, ws, gap0, a2f_tmp, dsumi, zsumi, &
                               delta, deltap, znorm, znormp, gp, gm
     USE ep_constants,  ONLY : ci, zero, czero, cone
     USE ep_constants,  ONLY : pi
@@ -650,14 +679,14 @@
         !
         i = iw + iwp
         IF (i <= nsw) THEN
-          esqrt = gp(iw, iwp) * a2f_iso(iwp) * znormp(i)
+          esqrt = gp(iw, iwp) * a2f_tmp(iwp) * znormp(i)
           znorm(iw) = znorm(iw) - esqrt * ws(i)
           delta(iw) = delta(iw) - esqrt * deltap(i)
         ENDIF
         !
         i = ABS(iw - iwp)
         IF (i > 0) THEN
-          esqrt = gm(iw, iwp) * a2f_iso(iwp) * znormp(i)
+          esqrt = gm(iw, iwp) * a2f_tmp(iwp) * znormp(i)
           znorm(iw) = znorm(iw) + esqrt * ws(i) * SIGN(1, iw - iwp)
           delta(iw) = delta(iw) + esqrt * deltap(i)
         ENDIF
@@ -912,7 +941,7 @@
     USE kinds,            ONLY : DP
     USE input,            ONLY : nqstep
     USE ep_constants,     ONLY : zero
-    USE supercond_common, ONLY : a2f_iso, wsph, dwsph
+    USE supercond_common, ONLY : a2f_tmp, wsph, dwsph
     !
     IMPLICIT NONE
     !
@@ -927,7 +956,7 @@
     !
     lambda_eph = zero
     DO iwph = 1, nqstep  ! loop over Omega (integration variable)
-      lambda_eph = lambda_eph + wsph(iwph) * a2f_iso(iwph) &
+      lambda_eph = lambda_eph + wsph(iwph) * a2f_tmp(iwph) &
                  / (wsph(iwph) * wsph(iwph) + omega * omega)
     ENDDO ! iwph
     lambda_eph = 2.d0 * lambda_eph * dwsph
@@ -1012,7 +1041,7 @@
     !!
     USE kinds, ONLY : DP
     USE input,         ONLY : nqstep
-    USE supercond_common,     ONLY : a2f_iso, wsph, dwsph
+    USE supercond_common,     ONLY : a2f_tmp, wsph, dwsph
     USE ep_constants,  ONLY : ci, czero
     !
     IMPLICIT NONE
@@ -1030,7 +1059,7 @@
     !
     lambda_eph = czero
     DO iwph = 1, nqstep  ! loop over Omega (integration variable)
-      lambda_eph = lambda_eph + wsph(iwph) * a2f_iso(iwph) &
+      lambda_eph = lambda_eph + wsph(iwph) * a2f_tmp(iwph) &
                  / (wsph(iwph) * wsph(iwph) - (omega - ci * omegap) * (omega - ci * omegap))
     ENDDO ! iwph
     lambda_eph = lambda_eph * 2.d0 * dwsph
@@ -1402,7 +1431,7 @@
     USE ep_constants,  ONLY : ci, eps6, zero, czero, one
     USE ep_constants,  ONLY : pi
     USE input,         ONLY : nqstep
-    USE supercond_common,     ONLY : a2f_iso, wsph, dwsph, ws, bewph, fdwp
+    USE supercond_common,     ONLY : a2f_tmp, wsph, dwsph, ws, bewph, fdwp
     !
     IMPLICIT NONE
     !
@@ -1484,8 +1513,8 @@
       g4 = e4 - ci * AIMAG(e4) + ci * pi * f4
       var1 = one - fdwp(iwp) + bewph(iwph)
       var2 = fdwp(iwp) + bewph(iwph)
-      kernelp = kernelp + a2f_iso(iwph) * (var1 * (g1 + g2) - var2 * (g3 + g4))
-      kernelm = kernelm + a2f_iso(iwph) * (var1 * (g1 - g2) + var2 * (g3 - g4))
+      kernelp = kernelp + a2f_tmp(iwph) * (var1 * (g1 + g2) - var2 * (g3 + g4))
+      kernelm = kernelm + a2f_tmp(iwph) * (var1 * (g1 - g2) + var2 * (g3 - g4))
     ENDDO ! iwph
     kernelp = kernelp * dwsph
     kernelm = kernelm * dwsph
@@ -1756,9 +1785,9 @@
     !!
     !----------------------------------------------------------------------
     !
-    USE input,         ONLY : fbw
-    USE supercond_common,     ONLY : wsi, wsn, deltai, deltaip, znormi, nznormi, &
-                              znormip, shifti, shiftip
+    USE input,            ONLY : fbw
+    USE supercond_common, ONLY : wsi, wsn, deltai, deltaip, znormi, nznormi, &
+                                 znormip, shifti, shiftip
     !
     IMPLICIT NONE
     !
@@ -1869,7 +1898,7 @@
     !!
     USE input,         ONLY : limag, fbw
     USE global_var,    ONLY : gtemp
-    USE supercond_common, ONLY : a2f_iso, wsph, nsiw, en, dosen
+    USE supercond_common, ONLY : a2f_tmp, wsph, nsiw, en, dosen
     !
     IMPLICIT NONE
     !
@@ -1882,8 +1911,8 @@
     ! read_a2f
     DEALLOCATE(wsph, STAT = ierr)
     IF (ierr /= 0) CALL errore('deallocate_iso', 'Error deallocating wsph', 1)
-    DEALLOCATE(a2f_iso, STAT = ierr)
-    IF (ierr /= 0) CALL errore('deallocate_iso', 'Error deallocating a2f_iso', 1)
+    DEALLOCATE(a2f_tmp, STAT = ierr)
+    IF (ierr /= 0) CALL errore('deallocate_iso', 'Error deallocating a2f_tmp', 1)
     ! eliashberg_grid
     IF (limag) THEN
       DEALLOCATE(nsiw, STAT = ierr)
