@@ -34,7 +34,7 @@ subroutine kcw_setup_screen
   USE control_kcw,       ONLY : alpha_final, iurho_wann, kcw_iverbosity, io_real_space, &
                                 read_unitary_matrix, num_wann, num_wann_occ, i_orb, iorb_start, &
                                 iorb_end, nqstot, occ_mat, l_do_alpha, group_alpha, &
-                                tmp_dir_kcw, tmp_dir_kcwq, x_q, lgamma_iq, io_sp,nrho, spin_component
+                                tmp_dir_kcw, tmp_dir_kcwq, x_q, lgamma_iq, io_sp, irr_bz, nrho, spin_component
   USE io_global,         ONLY : stdout
   USE klist,             ONLY : xk, nkstot, nelec, nelup, neldw
   USE cell_base,         ONLY : at, omega !, bg
@@ -226,6 +226,10 @@ subroutine kcw_setup_screen
     !
   ENDDO
   !
+  !read qlist_ibz
+  ! 
+  IF(irr_bz) CALL read_qlist_ibz()
+  !
   ! ... Which orbital to compute
   !
   iorb_from_input = .FALSE. 
