@@ -44,9 +44,6 @@ SUBROUTINE orthogonalize(dvpsi, evq, ikk, ikq, dpsi, npwq, dpsi_computed)
   USE control_lr,       ONLY : alpha_pv, nbnd_occ
   USE dfpt_tetra_mod,   ONLY : dfpt_tetra_beta
   USE two_chem,         ONLY : twochem
-#if defined(__CUDA)
-  USE cublas
-#endif
   !
   IMPLICIT NONE
   INTEGER, INTENT(IN) :: ikk, ikq   ! the index of the k and k+q points
@@ -90,10 +87,10 @@ SUBROUTINE orthogonalize(dvpsi, evq, ikk, ikq, dpsi, npwq, dpsi_computed)
      !
      !$acc host_data use_device(evq, dvpsi, ps)
      IF (noncolin) THEN
-        CALL zgemm( 'C', 'N', nbnd, nbnd_occ (ikk), npwx*npol, (1.d0,0.d0), &
+        CALL myzgemm( 'C', 'N', nbnd, nbnd_occ (ikk), npwx*npol, (1.d0,0.d0), &
              evq, npwx*npol, dvpsi, npwx*npol, (0.d0,0.d0), ps, nbnd )
      ELSE
-        CALL zgemm( 'C', 'N', nbnd, nbnd_occ (ikk), npwq, (1.d0,0.d0), &
+        CALL myzgemm( 'C', 'N', nbnd, nbnd_occ (ikk), npwq, (1.d0,0.d0), &
              evq, npwx, dvpsi, npwx, (0.d0,0.d0), ps, nbnd )
      END IF
      !$acc end host_data
@@ -192,12 +189,12 @@ SUBROUTINE orthogonalize(dvpsi, evq, ikk, ikq, dpsi, npwq, dpsi_computed)
      !
      !$acc host_data use_device(evq, dvpsi, ps)
      IF (noncolin) THEN
-        CALL zgemm( 'C', 'N',nbnd_occ(ikq), nbnd_occ(ikk), npwx*npol, &
+        CALL myzgemm( 'C', 'N',nbnd_occ(ikq), nbnd_occ(ikk), npwx*npol, &
              (1.d0,0.d0), evq, npwx*npol, dvpsi, npwx*npol, &
              (0.d0,0.d0), ps, nbnd )
      ELSEIF (gamma_only) THEN
         !$acc host_data use_device(ps_r)
-        CALL dgemm( 'C', 'N', nbnd_occ(ikq), nbnd_occ (ikk), 2*npwq, &
+        CALL mydgemm( 'C', 'N', nbnd_occ(ikq), nbnd_occ (ikk), 2*npwq, &
              2.0_DP, evq, 2*npwx, dvpsi, 2*npwx, &
              0.0_DP, ps_r, nbnd )
         IF (gstart == 2 ) THEN
@@ -206,7 +203,7 @@ SUBROUTINE orthogonalize(dvpsi, evq, ikk, ikq, dpsi, npwq, dpsi_computed)
         ENDIF
         !$acc end host_data
      ELSE
-        CALL zgemm( 'C', 'N', nbnd_occ(ikq), nbnd_occ (ikk), npwq, &
+        CALL myzgemm( 'C', 'N', nbnd_occ(ikq), nbnd_occ (ikk), npwq, &
              (1.d0,0.d0), evq, npwx, dvpsi, npwx, &
              (0.d0,0.d0), ps, nbnd )
      END IF
@@ -259,11 +256,11 @@ SUBROUTINE orthogonalize(dvpsi, evq, ikk, ikq, dpsi, npwq, dpsi_computed)
      !  metallic case
      !
      IF (noncolin) THEN
-        CALL zgemm( 'N', 'N', npwx*npol, nbnd_occ(ikk), nbnd, &
+        CALL myzgemm( 'N', 'N', npwx*npol, nbnd_occ(ikk), nbnd, &
              (1.d0,0.d0), dpsi, npwx*npol, ps, nbnd, (-1.0d0,0.d0), &
              dvpsi, npwx*npol )
      ELSE
-        CALL zgemm( 'N', 'N', npwq, nbnd_occ(ikk), nbnd, &
+        CALL myzgemm( 'N', 'N', npwq, nbnd_occ(ikk), nbnd, &
              (1.d0,0.d0), dpsi, npwx, ps, nbnd, (-1.0d0,0.d0), &
              dvpsi, npwx )
      END IF
@@ -273,15 +270,15 @@ SUBROUTINE orthogonalize(dvpsi, evq, ikk, ikq, dpsi, npwq, dpsi_computed)
      !  Insulators: note that nbnd_occ(ikk)=nbnd_occ(ikq) in an insulator
      !
      IF (noncolin) THEN
-        CALL zgemm( 'N', 'N', npwx*npol, nbnd_occ(ikk), nbnd_occ(ikk), &
+        CALL myzgemm( 'N', 'N', npwx*npol, nbnd_occ(ikk), nbnd_occ(ikk), &
              (1.d0,0.d0),dpsi,npwx*npol,ps,nbnd,(-1.0d0,0.d0), &
              dvpsi, npwx*npol )
      ELSEIF (gamma_only) THEN             
-        CALL ZGEMM( 'N', 'N', npwq, nbnd_occ(ikk), nbnd_occ(ikk), &
+        CALL myzgemm( 'N', 'N', npwq, nbnd_occ(ikk), nbnd_occ(ikk), &
              (1.d0,0.d0), dpsi, npwx, ps, nbnd, (-1.0d0,0.d0), &
              dvpsi, npwx )
      ELSE
-        CALL zgemm( 'N', 'N', npwq, nbnd_occ(ikk), nbnd_occ(ikk), &
+        CALL myzgemm( 'N', 'N', npwq, nbnd_occ(ikk), nbnd_occ(ikk), &
              (1.d0,0.d0), dpsi, npwx, ps, nbnd, (-1.0d0,0.d0), &
              dvpsi, npwx )
      END IF
