@@ -22,7 +22,7 @@ MODULE fft_wave
   !
   PRIVATE
   !
-  PUBLIC :: wave_r2g, wave_g2r, tgwave_r2g, tgwave_g2r
+  PUBLIC :: wave_r2g, wave_g2r, tgwave_r2g, tgwave_g2r, fwfft_wave, invfft_wave
   !
   !
 CONTAINS
@@ -252,5 +252,104 @@ CONTAINS
     !
   END SUBROUTINE tgwave_r2g
   !
+SUBROUTINE fwfft_wave (npwx, npw, igk, evc_g, evc_r)
+  !
+  !! Wave function FFT from R to G-space.
+  !
+  USE kinds,            ONLY : DP
+  USE fft_base,         ONLY : dffts
+  USE fft_interfaces,   ONLY : fwfft
+  USE noncollin_module, ONLY : noncolin, npol
+
+  IMPLICIT NONE
+  INTEGER, INTENT(IN) :: npwx, npw, igk(npw)
+  COMPLEX(DP), INTENT(INOUT) :: evc_g (npwx*npol), evc_r (dffts%nnr,npol)
+  !
+  INTEGER :: ig, ik
+
+#if defined(__CUDA) && defined(_OPENACC)
+  INTEGER, POINTER, DEVICE :: nl(:)
+  nl => dffts%nl_d
+#else
+  INTEGER, ALLOCATABLE :: nl(:)
+  ALLOCATE( nl(dffts%ngm) )
+  nl = dffts%nl
+#endif
+
+  !$acc host_data use_device(evc_r)
+  CALL fwfft ('Wave', evc_r(:,1), dffts)
+  !$acc end host_data
+  !$acc parallel loop private(ik)
+  DO ig = 1, npw
+     ik = nl(igk(ig))
+     evc_g (ig) = evc_g (ig) + evc_r (ik,1)
+  ENDDO
+  IF (noncolin) THEN
+     !$acc host_data use_device(evc_r)
+     CALL fwfft ('Wave', evc_r(:,2), dffts)
+     !$acc end host_data
+     !$acc parallel loop private(ik) 
+     DO ig = 1, npw
+        ik = nl(igk(ig))
+        evc_g (ig+npwx) = evc_g (ig+npwx) + evc_r (ik,2)
+     ENDDO
+  ENDIF
+
+#if !defined(__CUDA) || !defined(_OPENACC)
+  DEALLOCATE(nl)
+#endif
+END SUBROUTINE fwfft_wave
+  !
+SUBROUTINE invfft_wave (npwx, npw, igk, evc_g, evc_r)
+  !
+  !! Wave function FFT from G to R-space.
+  !
+  USE kinds,            ONLY : DP
+  USE fft_base,         ONLY : dffts
+  USE fft_interfaces,   ONLY : invfft
+  USE noncollin_module, ONLY : noncolin, npol
+
+  IMPLICIT NONE
+  INTEGER, INTENT(IN) :: npwx, npw, igk(npw)
+  COMPLEX(DP), INTENT(IN) :: evc_g (npwx*npol)
+  COMPLEX(DP), INTENT(OUT):: evc_r (dffts%nnr,npol)
+  !
+  INTEGER :: ig, ik
+
+#if defined(__CUDA) && defined(_OPENACC)
+  INTEGER, POINTER, DEVICE :: nl(:)
+  nl => dffts%nl_d
+#else
+  INTEGER, ALLOCATABLE :: nl(:)
+  ALLOCATE( nl(dffts%ngm) )
+  nl = dffts%nl
+#endif
+
+  !$acc kernels
+  evc_r(:,:) = (0.0_dp, 0.0_dp)
+  !$acc end kernels
+  !$acc parallel loop private(ik)
+  DO ig = 1, npw
+     ik = nl(igk(ig))
+     evc_r (ik, 1) = evc_g (ig)
+  ENDDO
+  !$acc host_data use_device(evc_r)
+  CALL invfft ('Wave', evc_r(:,1), dffts)
+  !$acc end host_data
+  IF (noncolin) THEN
+     !$acc parallel loop private(ik)
+     DO ig = 1, npw
+        ik = nl(igk(ig))
+        evc_r (ik, 2) = evc_g (ig+npwx)
+     ENDDO
+     !$acc host_data use_device(evc_r)
+     CALL invfft ('Wave', evc_r(:,2), dffts)
+     !$acc end host_data
+  ENDIF
+
+#if !defined(__CUDA) || !defined(_OPENACC)
+  DEALLOCATE(nl)
+#endif
+END SUBROUTINE invfft_wave
   !
 END MODULE fft_wave
