@@ -153,12 +153,12 @@ MODULE qexsd_input
      END SELECT
      ALLOCATE (inpOcc_objs(inpOcc_size))
      IF ( inpOcc_size .GT. 1) THEN
-        CALL qes_init ( inpOcc_objs(1),"input_occupations", ISPIN = 1, &
+        CALL qes_init ( inpOcc_objs(1),"inputOccupations", ISPIN = 1, &
                   SPIN_FACTOR = 1._DP, INPUTOCCUPATIONS = input_occupations(1:nbnd) )
-        CALL qes_init ( inpOcc_objs(2),"input_occupations", 2, &
+        CALL qes_init ( inpOcc_objs(2),"inputOccupations", 2, &
                   SPIN_FACTOR = 1._DP , INPUTOCCUPATIONS = input_occupations_minority(1:nbnd))
      ELSE
-        CALL qes_init ( inpOcc_objs(1),"input_occupations", ISPIN = 1, SPIN_FACTOR = 2._DP , &
+        CALL qes_init ( inpOcc_objs(1),"inputOccupations", ISPIN = 1, SPIN_FACTOR = 2._DP , &
                                                                  INPUTOCCUPATIONS = input_occupations(1:nbnd) )
      END IF
   END IF
@@ -227,7 +227,7 @@ MODULE qexsd_input
                                           conv_thr, mixing_ndim, exx_nstep, max_nstep, tqr, real_space, &
                                           tq_smoothing, tbeta_smoothing, & 
                                           diago_thr_init, diago_full_acc, &
-                                          diago_cg_maxiter, diago_ppcg_maxiter, diago_david_ndim, &
+                                          diago_cg_maxiter, diago_david_ndim, &
                                           diago_rmm_ndim, diago_rmm_conv, diago_gs_nblock)
   !-------------------------------------------------------------------------------------------
   !
@@ -237,7 +237,7 @@ MODULE qexsd_input
   CHARACTER(LEN=*),INTENT(IN)             :: diagonalization,mixing_mode
   REAL(DP),INTENT(IN)                     :: mixing_beta, conv_thr, diago_thr_init
   INTEGER,INTENT(IN)                      :: mixing_ndim,exx_nstep, max_nstep, diago_cg_maxiter, &
-                                             diago_ppcg_maxiter, diago_david_ndim, &
+                                             diago_david_ndim, &
 
                                              diago_rmm_ndim, diago_gs_nblock
   LOGICAL,OPTIONAL,INTENT(IN)             :: diago_full_acc,tqr, real_space, tq_smoothing, tbeta_smoothing, &
@@ -251,7 +251,6 @@ MODULE qexsd_input
                                 TQ_SMOOTHING= tq_smoothing, TBETA_SMOOTHING = tbeta_smoothing,& 
                                 REAL_SPACE_Q=tqr, REAL_SPACE_BETA = real_space, DIAGO_THR_INIT=diago_thr_init,& 
                                 DIAGO_FULL_ACC=diago_full_acc,DIAGO_CG_MAXITER=diago_cg_maxiter, &
-                                DIAGO_PPCG_MAXITER=diago_ppcg_maxiter, &
                                 DIAGO_RMM_NDIM=diago_rmm_ndim, DIAGO_RMM_CONV=diago_rmm_conv, &
                                 DIAGO_GS_NBLOCK=diago_gs_nblock)
    !
@@ -259,7 +258,7 @@ MODULE qexsd_input
    !
    !
    !-------------------------------------------------------------------------------------------------
-   SUBROUTINE qexsd_init_k_points_ibz(obj,k_points,calculation,nk1,nk2,nk3,s1,s2,s3,nk,alat,a1, ibrav_lattice,xk,wk)
+   SUBROUTINE qexsd_init_k_points_ibz(obj,k_points,calculation,nk1,nk2,nk3,s1,s2,s3,nk,alat,a1, ibrav_lattice,xk,wk,labelk)
    ! 
    IMPLICIT NONE
    ! 
@@ -267,6 +266,7 @@ MODULE qexsd_input
    CHARACTER(LEN=*),INTENT(IN)          :: k_points,calculation
    INTEGER,INTENT(IN)                   :: nk1,nk2,nk3,s1,s2,s3,nk
    REAL(DP),INTENT(IN), OPTIONAL        :: xk(:,:),wk(:)
+   CHARACTER(LEN=*),INTENT(IN), OPTIONAL:: labelk(:)
    REAL(DP),INTENT(IN)                  :: alat,a1(3)
    LOGICAL,INTENT(IN)                   :: ibrav_lattice
    !
@@ -281,6 +281,7 @@ MODULE qexsd_input
    REAL(DP)                             :: scale_factor
    INTEGER, POINTER                     :: kdim_opt => NULL()
    INTEGER, TARGET                      :: kdim 
+   LOGICAL                              :: has_kpt_label
    !
   
    IF (TRIM(k_points).EQ."automatic") THEN 
@@ -320,7 +321,19 @@ MODULE qexsd_input
           ALLOCATE  (kp_obj(kdim))      
           DO ik=1,kdim
              my_xk=xk(:,ik)*scale_factor
-             CALL qes_init (kp_obj(ik),"k_point", WEIGHT = wk(ik),K_POINT=my_xk)
+             !
+             has_kpt_label = .FALSE.
+             IF ( PRESENT(labelk) ) THEN
+                   IF (TRIM(labelk(ik)) /= '') THEN
+                      has_kpt_label = .TRUE.
+                      ! Create with custom K point label, if present
+                      CALL qes_init (kp_obj(ik),"k_point",WEIGHT = wk(ik), LABEL=TRIM(labelk(ik)),K_POINT=my_xk)
+                   END IF
+             END IF
+             !
+             IF (.NOT. has_kpt_label) &
+                CALL qes_init (kp_obj(ik),"k_point",WEIGHT = wk(ik),K_POINT=my_xk)
+             !
           END DO
       END IF
    END IF    
