@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2023 Quantum ESPRESSO group
+! Copyright (C) 2001-2025 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -142,7 +142,7 @@ SUBROUTINE electrons()
            CALL v_of_rho( rho, rho_core, rhog_core, &
                ehart, etxc, vtxc, eth, etotefield, charge, v)
            IF (lrism) CALL rism_calc3d(rho%of_g(:, 1), esol, vsol, v%of_r, tr2)
-           IF (okpaw) CALL PAW_potential(rho%bec, ddd_PAW, epaw,etot_cmp_paw)
+           IF (okpaw) CALL PAW_potential(rho%bec, ddd_paw, epaw,etot_cmp_paw)
            CALL set_vrs( vrs, vltot, v%of_r, kedtau, v%kin_r, dfftp%nnr, &
                          nspin, doublegrid )
            !
@@ -238,7 +238,7 @@ SUBROUTINE electrons()
         !
         IF (lrism) CALL rism_calc3d(rho%of_g(:, 1), esol, vsol, v%of_r, tr2)
         !
-        IF (okpaw) CALL PAW_potential(rho%bec, ddd_PAW, epaw,etot_cmp_paw)
+        IF (okpaw) CALL PAW_potential(rho%bec, ddd_paw, epaw,etot_cmp_paw)
         CALL set_vrs( vrs, vltot, v%of_r, kedtau, v%kin_r, dfftp%nnr, &
              nspin, doublegrid )
         !
@@ -425,7 +425,7 @@ SUBROUTINE electrons_scf ( printout, exxen )
   USE ldaU,                 ONLY : eth, lda_plus_u, lda_plus_u_kind, &
                                    niter_with_fixed_ns, hub_pot_fix, &
                                    nsg, nsgnew, v_nsg, at_sc, neighood, &
-                                   ldim_u, is_hubbard_back
+                                   ldim_u, is_hubbard_back, apply_U, orbital_resolved
   USE extfield,             ONLY : tefield, etotefield, gate, etotgatefield !TB
   USE noncollin_module,     ONLY : noncolin, magtot_nc, i_cons,  bfield, &
                                    lambda, report, domag, nspin_mag, npol
@@ -446,7 +446,6 @@ SUBROUTINE electrons_scf ( printout, exxen )
   USE paw_onecenter,        ONLY : PAW_potential
   USE paw_symmetry,         ONLY : PAW_symmetrize_ddd
   USE dfunct,               ONLY : newd
-  USE dfunct_gpum,          ONLY : newd_gpu
   USE esm,                  ONLY : do_comp_esm, esm_printpot, esm_ewald
   USE gcscf_module,         ONLY : lgcscf, gcscf_mu, gcscf_ignore_mun, gcscf_set_nelec
   USE clib_wrappers,        ONLY : memstat
@@ -624,6 +623,11 @@ SUBROUTINE electrons_scf ( printout, exxen )
      !
      IF ( iter > 1 ) THEN
         !
+#if defined (__OSCDFT)
+        IF (use_oscdft .AND. (oscdft_ctx%inp%oscdft_type==2)) THEN
+           ! do nothing
+        ELSE
+#endif
         IF ( iter == 2 ) THEN
            !
            IF ( lgcscf ) THEN
@@ -633,8 +637,18 @@ SUBROUTINE electrons_scf ( printout, exxen )
            ENDIF
            !
         ENDIF
+#if defined (__OSCDFT)
+        ENDIF
+#endif
         !
-        ethr = MIN( ethr, 0.1D0*dr2 / MAX( 1.D0, nelec ) )
+        IF ( orbital_resolved ) THEN
+           ethr = MIN( ethr, 0.05D0*dr2 / MAX( 1.D0, nelec ) )
+           ! ... tighten diagonalization in orbital-resolved DFT+U
+           ! ... for faster convergence
+        ELSE
+           !
+           ethr = MIN( ethr, 0.1D0*dr2 / MAX( 1.D0, nelec ) )
+        ENDIF
         ! ... do not allow convergence threshold to become too small:
         ! ... iterative diagonalization may become unstable
         ethr = MAX( ethr, 1.D-13 )
@@ -665,7 +679,7 @@ SUBROUTINE electrons_scf ( printout, exxen )
         ! ... diagonalization of the KS hamiltonian
         !
 #if defined (__OSCDFT)
-        IF (use_oscdft) THEN
+        IF (use_oscdft .AND. (oscdft_ctx%inp%oscdft_type==1)) THEN
            CALL oscdft_electrons(oscdft_ctx, iter, et, nbnd, nkstot, nks)
         ELSE
 #endif
@@ -761,9 +775,16 @@ SUBROUTINE electrons_scf ( printout, exxen )
            !
            IF (hub_pot_fix) THEN
              IF (lda_plus_u_kind.EQ.0) THEN
-                IF (noncolin) CALL errore('electrons_scf', &
-                & 'hub_pot_fix is not implemented for (lda_plus_u_kind=0 .AND. noncolin)',1)     
-                rho%ns = rhoin%ns ! back to input values
+                IF (noncolin) THEN
+                   ! call occupation counting routine before resetting ns
+                   IF (orbital_resolved) CALL alpha_m_nc_trace(rho%ns_nc) 
+                   rho%ns_nc = rhoin%ns_nc
+                ELSE
+                   ! call occupation counting routine before resetting ns
+                   IF (orbital_resolved) CALL alpha_m_trace(rho%ns) 
+                   rho%ns = rhoin%ns ! back to input values
+                ENDIF
+                !
                 IF (lhb) rho%nsb = rhoin%nsb
              ELSEIF (lda_plus_u_kind.EQ.1) THEN
                 CALL errore('electrons_scf', &
@@ -880,7 +901,7 @@ SUBROUTINE electrons_scf ( printout, exxen )
                                &    "too large:",/,5X,                      &
                                & "Diagonalizing with lowered threshold",/)' )
               !
-              ethr = 0.1D0*dr2 / MAX( 1.D0, nelec )
+              ethr = MAX( 0.1D0*dr2 / MAX( 1.D0, nelec ), 1.D-13 )
               !
               CYCLE scf_step
               !
@@ -893,6 +914,26 @@ SUBROUTINE electrons_scf ( printout, exxen )
            ! ... no convergence yet: calculate new potential from mixed
            ! ... charge density (i.e. the new estimate)
            !
+           IF ( orbital_resolved .AND. (.NOT. apply_U) ) THEN
+              !
+              IF ( ethr .LE. 1.D-4 .OR. iter .GT. 10 ) THEN
+                 !
+                 ! ... activate the orbital-resolved Hubbard corrections
+                 ! ... once the eigenstates (and thus, the eigenvectors) 
+                 ! ... have stablilized. Also turn them on after 10 iterations
+                 ! ... but print a warning message in this case.
+                 WRITE( stdout, '(/,5X,47("="))')
+                 WRITE( stdout, '(/,5X,"Switching ON", &
+                    & " orbital-resolved Hubbard corrections")' )
+                 IF ( iter .GT. 10 ) &
+                    WRITE( stdout, '(/,5X,"WARNING: check convergence of eigenstates")')
+                 WRITE( stdout, '(/,5X,47("="))')
+                 !
+                 apply_U = .TRUE.
+              ENDIF
+           ENDIF
+           !
+           !
            CALL v_of_rho( rhoin, rho_core, rhog_core, &
                           ehart, etxc, vtxc, eth, etotefield, charge, v )
            !
@@ -918,6 +959,18 @@ SUBROUTINE electrons_scf ( printout, exxen )
            !
            CALL scf_type_COPY( rhoin, rho )
            !
+#if defined (__OSCDFT)
+           IF (use_oscdft .AND. (oscdft_ctx%inp%oscdft_type==2)) THEN
+              IF (lda_plus_u .AND. .NOT.oscdft_ctx%conv) THEN
+                 IF (lda_plus_u_kind.EQ.0) THEN
+                    CALL write_ns()
+                 ELSEIF (lda_plus_u_kind.EQ.2) THEN
+                    CALL write_nsg()
+                 ENDIF
+              ENDIF
+           ENDIF
+#endif
+           !
            IF (lda_plus_u .AND. lda_plus_u_kind.EQ.2) nsgnew = nsg
            !
            IF ( lgcscf ) THEN
@@ -975,7 +1028,7 @@ SUBROUTINE electrons_scf ( printout, exxen )
      END IF
 #endif
 #if defined (__OSCDFT)
-     IF (use_oscdft) THEN
+     IF (use_oscdft .AND. (oscdft_ctx%inp%oscdft_type==1)) THEN
         CALL oscdft_scf_energy(oscdft_ctx, plugin_etot)
      END IF
 #endif
@@ -989,8 +1042,7 @@ SUBROUTINE electrons_scf ( printout, exxen )
      ! ... term in the nonlocal potential
      ! ... PAW: newd contains PAW updates of NL coefficients
      !
-     IF (.not. use_gpu) CALL newd()
-     IF (      use_gpu) CALL newd_gpu()
+     CALL newd()
      !
      IF ( lelfield ) en_el =  calc_pol ( )
      !
@@ -1021,7 +1073,11 @@ SUBROUTINE electrons_scf ( printout, exxen )
            ! needed when computing U (and V) in a SCF way
            IF (hub_pot_fix) THEN
               IF (lda_plus_u_kind.EQ.0) THEN
-                 CALL new_ns(rho%ns)
+                 IF (noncolin) THEN
+                    CALL new_ns_nc(rho%ns_nc)
+                 ELSE
+                    CALL new_ns(rho%ns)
+                 ENDIF
                  IF (lhb) CALL new_nsb(rho%nsb)
               ELSEIF (lda_plus_u_kind.EQ.2) THEN
                  CALL new_nsg()
@@ -1030,9 +1086,13 @@ SUBROUTINE electrons_scf ( printout, exxen )
            !
            ! Write the occupation matrices
            IF (lda_plus_u_kind == 0) THEN
-              IF (noncolin) THEN      
+              IF (noncolin) THEN
+                 IF ( orbital_resolved .AND. hub_pot_fix ) &
+                    CALL alpha_m_nc_trace(rho%ns_nc)      
                  CALL write_ns_nc()
-              ELSE        
+              ELSE
+                 IF ( orbital_resolved .AND. hub_pot_fix ) &
+                    CALL alpha_m_trace(rho%ns)
                  CALL write_ns()
               ENDIF
            ELSEIF (lda_plus_u_kind == 1) THEN
@@ -1051,7 +1111,7 @@ SUBROUTINE electrons_scf ( printout, exxen )
            !
         ENDIF
 #if defined (__OSCDFT)
-        IF (use_oscdft) THEN
+        IF (use_oscdft .AND. (oscdft_ctx%inp%oscdft_type==1)) THEN
            CALL oscdft_print_ns(oscdft_ctx)
         END IF
 #endif
@@ -1742,7 +1802,8 @@ SUBROUTINE electrons_scf ( printout, exxen )
        IF (use_environ) CALL print_environ_energies('PW')
 #endif
 #if defined (__OSCDFT)
-       IF (use_oscdft .AND. conv_elec) CALL oscdft_print_energies(oscdft_ctx)
+       IF (use_oscdft .AND. conv_elec .AND. (oscdft_ctx%inp%oscdft_type==1)) &
+          CALL oscdft_print_energies(oscdft_ctx)
 #endif
        !
        IF ( lsda ) WRITE( stdout, 9017 ) magtot, absmag
