@@ -233,7 +233,7 @@ SUBROUTINE forces()
   IF (use_environ) CALL calc_environ_force(force)
 #endif
 #if defined (__OSCDFT)
-  IF (use_oscdft) CALL oscdft_apply_forces(oscdft_ctx)
+  IF (use_oscdft .AND. (oscdft_ctx%inp%oscdft_type==1)) CALL oscdft_apply_forces(oscdft_ctx)
 #endif
   !
   ! ... Berry's phase electric field terms
@@ -443,7 +443,7 @@ SUBROUTINE forces()
      !
   END IF
 #if defined (__OSCDFT)
-  IF (use_oscdft) CALL oscdft_print_forces(oscdft_ctx)
+  IF (use_oscdft .AND. (oscdft_ctx%inp%oscdft_type==1)) CALL oscdft_print_forces(oscdft_ctx)
 #endif
   !
   sumfor = 0.D0
