@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2007 PWSCF group
+! Copyright (C) 2001-2024 Quantum ESPRESSO Foundation
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -18,7 +18,4 @@ MODULE g_psi_mod
   REAL(DP), ALLOCATABLE :: s_diag(:,:)
   !! diagonal part of the overlap matrix
   !
-#if defined(__CUDA)
-    attributes(PINNED) :: h_diag, s_diag
-#endif 
 END MODULE g_psi_mod
