@@ -22,7 +22,12 @@ MODULE oscdft_forces_subs
 
          IMPLICIT NONE
 
-         TYPE(oscdft_context_type), INTENT(INOUT) :: ctx
+         TYPE(oscdft_context_type), INTENT(INOUT), TARGET :: ctx
+         TYPE(oscdft_input_type),   POINTER       :: inp
+
+         inp => ctx%inp
+
+         IF (.NOT.(inp%oscdft_type==1)) RETURN
 
          IF (.NOT.ctx%inp%skip_forces .AND. ctx%idx%nconstr > 0) THEN
             IF (.NOT. ALLOCATED(ctx%force_oscdft)) ALLOCATE(ctx%force_oscdft(3,nat))
@@ -37,9 +42,14 @@ MODULE oscdft_forces_subs
          USE ions_base, ONLY : nat, ityp
          IMPLICIT NONE
 
-         TYPE(oscdft_context_type), INTENT(INOUT) :: ctx
+         TYPE(oscdft_context_type), INTENT(INOUT), TARGET :: ctx
+         TYPE(oscdft_input_type),   POINTER       :: inp
          INTEGER                                  :: na
 
+         inp => ctx%inp
+
+         IF (.NOT.(inp%oscdft_type==1)) RETURN
+
          IF (.NOT.ctx%inp%skip_forces .AND. ctx%idx%nconstr > 0) THEN
             WRITE(stdout, '(/,5x, "OS-CDFT contribution to forces:")')
             DO na=1,nat
