@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2004-2015 Quantum ESPRESSO group
+! Copyright (C) 2004-2024 Quantum ESPRESSO Foundation
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -2210,16 +2210,14 @@ MODULE realus
     !
     INTEGER :: ebnd
     !
-    !Task groups
-    !
-    !The new task group version based on vloc_psi
     !print *, "->Real space"
     !
-    !$acc data present_or_copyin(orbital) present_or_copyout(psic)
-    !
     CALL start_clock( 'invfft_orbital' )
     !
+    !$acc data present_or_copyin(orbital) present_or_copyout(psic)
     IF( dffts%has_task_groups ) THEN
+        !
+        ! Task groups - based on vloc_psi
         !
         CALL tgwave_g2r( orbital(1:ngk(1),ibnd:last), tg_psic, dffts, ngk(1) )
         !
@@ -2230,7 +2228,9 @@ MODULE realus
           ENDIF
         ENDIF
         !
-    ELSE !Task groups not used
+     ELSE
+        !
+        ! No Task groups
         !
         ebnd = ibnd
         IF ( ibnd < last ) ebnd = ebnd + 1
@@ -2239,17 +2239,17 @@ MODULE realus
         !
         IF (PRESENT(conserved)) THEN
           IF (conserved) THEN
+             call errore('invfft_orbital_gamma','unverified case',1)
             IF (.NOT. ALLOCATED(psic_temp) ) ALLOCATE( psic_temp(SIZE(psic)) )
             CALL zcopy( SIZE(psic), psic, 1, psic_temp, 1 )
           ENDIF
         ENDIF
         !
     ENDIF
+    !$acc end data
     !
     CALL stop_clock( 'invfft_orbital' )
     !
-    !$acc end data
-    !
   END SUBROUTINE invfft_orbital_gamma
   !
   !--------------------------------------------------------------------------
@@ -2307,6 +2307,7 @@ MODULE realus
         !
         incr = 2*fftx_ntgrp(dffts)
         ALLOCATE( psio(ngk(1),incr) )
+        !$acc enter data create(psio)
         !
         brange = last-ibnd+1
         !
@@ -2334,6 +2335,7 @@ MODULE realus
            ENDIF
         ENDDO
         !
+        !$acc exit data delete(psio)
         DEALLOCATE( psio )
         !
         IF (PRESENT(conserved)) THEN
@@ -2350,6 +2352,7 @@ MODULE realus
         brange = ebnd-ibnd+1
         !
         ALLOCATE( psio(ngk(1),brange) )
+        !$acc enter data create(psio)
         !
         CALL wave_r2g( psic(1:dffts%nnr), psio, dffts )
         !
@@ -2378,6 +2381,7 @@ MODULE realus
            ENDDO
         ENDIF
         !
+        !$acc exit data delete(psio)
         DEALLOCATE( psio )
         !
         IF (PRESENT(conserved)) THEN
@@ -2427,13 +2431,10 @@ MODULE realus
     !! if this flag is true, the orbital is stored in temporary memory
     !
     INTEGER :: ik_
-!-------------------TEMPORARY-----------
-    INTEGER :: ngk1
-    LOGICAL :: is_present, acc_is_present
-!---------------------------------------    
     !
     CALL start_clock( 'invfft_orbital' )
     !
+    !$acc data present_or_copyin(orbital) present_or_copyout(psic)
     ! ... current_k  variable  must contain the index of the desired kpoint
     ik_ = current_k ; IF (PRESENT(ik)) ik_ = ik
     !
@@ -2452,22 +2453,16 @@ MODULE realus
        !
        CALL wave_g2r( orbital(:,ibnd:ibnd), psic, dffts, igk=igk_k(:,ik_) )
        !
-!-------------TEMPORARY---------------------
-       ngk1 = SIZE(psic)
-#if defined(_OPENACC)
-       is_present = acc_is_present(psic,ngk1)
-       !$acc update self(psic) if (is_present)
-#endif
-!-------------------------------------------       
-       !
        IF (PRESENT(conserved)) THEN
           IF (conserved) THEN
+             call errore('invfft_orbital_k','unverified case',1)
              IF (.NOT. ALLOCATED(psic_temp) ) ALLOCATE( psic_temp(SIZE(psic)) )
              psic_temp = psic
           ENDIF
        ENDIF
        !
     ENDIF
+    !$acc end data
     !
     CALL stop_clock( 'invfft_orbital' )
     !
@@ -2515,13 +2510,11 @@ MODULE realus
     INTEGER :: idx, ik_ , incr, ig, brange
     LOGICAL :: add_to_orbital_
     COMPLEX(DP), ALLOCATABLE :: psio(:,:)
-!-------------------TEMPORARY-----------
-    INTEGER :: ngk1
-    LOGICAL :: is_present, acc_is_present
-!---------------------------------------    
     !
     CALL start_clock( 'fwfft_orbital' )
     !
+    !$acc data present_or_copyin(psic) present_or_copy(orbital)
+    !
     add_to_orbital_=.FALSE. ; IF( PRESENT(add_to_orbital)) add_to_orbital_ = add_to_orbital
     !
     ! ... current_k  variable  must contain the index of the desired kpoint
@@ -2533,6 +2526,7 @@ MODULE realus
        incr = fftx_ntgrp(dffts)
        !
        ALLOCATE( psio(ngk(ik_),incr) )
+       !$acc enter data create(psio)
        !
        brange = last-ibnd+1
        !
@@ -2550,6 +2544,7 @@ MODULE realus
           !
        ENDDO
        !
+       !$acc exit data delete(psio)
        DEALLOCATE( psio )
        !
        IF (PRESENT(conserved)) THEN
@@ -2561,31 +2556,23 @@ MODULE realus
     ELSE !non task groups version
        !
        ALLOCATE( psio(ngk(ik_),1) )
+       !$acc enter data create(psio)
        !
        CALL wave_r2g( psic(1:dffts%nnr), psio, dffts, igk=igk_k(:,ik_) )
-       !
-       !-------------TEMPORARY---------------------
-       ngk1 = SIZE(psic)
-#if defined(_OPENACC)
-       is_present = acc_is_present(psic,ngk1)
-       !$acc update self(psic) if (is_present)
-#endif
-!-------------------------------------------   
        !
        IF( add_to_orbital_ ) THEN
-          !$omp parallel do default(shared) private(ig)
+          !$acc parallel loop
           DO ig = 1, ngk(ik_)
              orbital(ig,ibnd) = orbital(ig,ibnd) + psio(ig,1)
           ENDDO
-          !$omp end parallel do
        ELSE
-          !$omp parallel do default(shared) private(ig)
+          !$acc parallel loop
           DO ig = 1, ngk(ik_)
              orbital(ig,ibnd) = psio(ig,1)
           ENDDO
-          !$omp end parallel do
        ENDIF
        !
+       !$acc exit data delete(psio)
        DEALLOCATE( psio )
        !
        IF ( PRESENT(conserved) ) THEN
@@ -2595,6 +2582,8 @@ MODULE realus
        ENDIF
     ENDIF
     !
+    !$acc end data
+    !
     CALL stop_clock( 'fwfft_orbital' )
     !
   END SUBROUTINE fwfft_orbital_k
